@@ -1,3 +1,9 @@
+#
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
+# License: http://pysnmp.sf.net/license.html
+#
 import sys
 from pyasn1.compat.octets import null
 from pysnmp.proto import rfc3411, error
@@ -7,11 +13,8 @@ from pysnmp import debug
 
 # 3.4
 class NotificationReceiver:
-    pduTypes = (
-        v1.TrapPDU.tagSet,        
-        v2c.SNMPv2TrapPDU.tagSet,
-        v2c.InformRequestPDU.tagSet
-        )
+    pduTypes = (v1.TrapPDU.tagSet, v2c.SNMPv2TrapPDU.tagSet,
+                v2c.InformRequestPDU.tagSet)
 
     def __init__(self, snmpEngine, cbFun, cbCtx=None):
         snmpEngine.msgAndPduDsp.registerContextEngineId(
@@ -27,20 +30,10 @@ class NotificationReceiver:
         )
         self.__cbFun = self.__cbCtx = None
 
-    def processPdu(
-        self,
-        snmpEngine,
-        messageProcessingModel,
-        securityModel,
-        securityName,
-        securityLevel,
-        contextEngineId,
-        contextName,
-        pduVersion,
-        PDU,
-        maxSizeResponseScopedPDU,
-        stateReference
-        ):
+    def processPdu(self, snmpEngine, messageProcessingModel,
+                   securityModel, securityName, securityLevel,
+                   contextEngineId, contextName, pduVersion, PDU,
+                   maxSizeResponseScopedPDU, stateReference):
 
         # Agent-side API complies with SMIv2
         if messageProcessingModel == 0:
@@ -49,17 +42,18 @@ class NotificationReceiver:
         else:
             origPdu = None
 
-        errorStatus = 'noError'; errorIndex = 0
+        errorStatus = 'noError'
+        errorIndex = 0
         varBinds = v2c.apiPDU.getVarBinds(PDU)
 
         debug.logger & debug.flagApp and debug.logger('processPdu: stateReference %s, varBinds %s' % (stateReference, varBinds))
-        
+
         # 3.4
         if PDU.tagSet in rfc3411.confirmedClassPDUs:
             # 3.4.1 --> no-op
-            
+
             rspPDU = v2c.apiPDU.getResponse(PDU)
-            
+
             # 3.4.2
             v2c.apiPDU.setErrorStatus(rspPDU, errorStatus)
             v2c.apiPDU.setErrorIndex(rspPDU, errorIndex)
@@ -76,23 +70,15 @@ class NotificationReceiver:
             # 3.4.3
             try:
                 snmpEngine.msgAndPduDsp.returnResponsePdu(
-                    snmpEngine,
-                    messageProcessingModel,
-                    securityModel,
-                    securityName,
-                    securityLevel,
-                    contextEngineId,
-                    contextName,
-                    pduVersion,
-                    rspPDU,
-                    maxSizeResponseScopedPDU,
-                    stateReference,
-                    statusInformation
-                    )
+                    snmpEngine, messageProcessingModel, securityModel,
+                    securityName, securityLevel, contextEngineId,
+                    contextName, pduVersion, rspPDU, maxSizeResponseScopedPDU,
+                    stateReference, statusInformation)
+
             except error.StatusInformation:
                 debug.logger & debug.flagApp and debug.logger('processPdu: stateReference %s, statusInformation %s' % (stateReference, sys.exc_info()[1]))
                 snmpSilentDrops, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('__SNMPv2-MIB', 'snmpSilentDrops')
-                snmpSilentDrops.syntax = snmpSilentDrops.syntax + 1
+                snmpSilentDrops.syntax += 1
 
         elif PDU.tagSet in rfc3411.unconfirmedClassPDUs:
             pass
@@ -102,20 +88,14 @@ class NotificationReceiver:
         debug.logger & debug.flagApp and debug.logger('processPdu: stateReference %s, user cbFun %s, cbCtx %s, varBinds %s' % (stateReference, self.__cbFun, self.__cbCtx, varBinds))
 
         if self.__cbFunVer:
-            self.__cbFun(
-                snmpEngine, stateReference, contextEngineId, contextName,
-                varBinds, self.__cbCtx
-                )
+            self.__cbFun(snmpEngine, stateReference, contextEngineId,
+                         contextName, varBinds, self.__cbCtx)
         else:
             # Compatibility stub (handle legacy cbFun interface)
             try:
-                self.__cbFun(
-                    snmpEngine, contextEngineId, contextName,
-                    varBinds, self.__cbCtx
-                    )
+                self.__cbFun(snmpEngine, contextEngineId, contextName,
+                             varBinds, self.__cbCtx)
             except TypeError:
                 self.__cbFunVer = 1
-                self.__cbFun(
-                    snmpEngine, stateReference, contextEngineId, contextName,
-                    varBinds, self.__cbCtx
-                    )
+                self.__cbFun(snmpEngine, stateReference, contextEngineId,
+                             contextName, varBinds, self.__cbCtx)
