@@ -1,3 +1,9 @@
+#
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
+# License: http://pysnmp.sf.net/license.html
+#
 from pysnmp.proto import error
 
 class Cache:
@@ -27,5 +33,5 @@ class Cache:
         for index, cachedParams in list(self.__cacheRepository.items()):
             if cbFun:
                 if cbFun(index, cachedParams, cbCtx):
-                    del self.__cacheRepository[index]                    
-
+                    if index in self.__cacheRepository:
+                        del self.__cacheRepository[index]
