@@ -1,3 +1,9 @@
+#
+# This file is part of pysnmp software.
+#
+# Copyright (c) 2005-2016, Ilya Etingof <ilya@glas.net>
+# License: http://pysnmp.sf.net/license.html
+#
 import sys
 import traceback
 from pysnmp.smi.indices import OidOrderedDict
@@ -6,14 +12,16 @@ from pysnmp.proto import rfc1902
 from pysnmp import cache, debug
 from pyasn1.error import PyAsn1Error
 
-( Integer, ObjectIdentifier ) = mibBuilder.importSymbols(
+Integer, ObjectIdentifier = mibBuilder.importSymbols(
     "ASN1", "Integer", "ObjectIdentifier"
-    )
-( ConstraintsIntersection, ConstraintsUnion, SingleValueConstraint,
-  ValueRangeConstraint, ValueSizeConstraint, ) = mibBuilder.importSymbols(
-    "ASN1-REFINEMENT", "ConstraintsIntersection", "ConstraintsUnion",
-    "SingleValueConstraint", "ValueRangeConstraint", "ValueSizeConstraint"
-    )
+)
+
+(ConstraintsIntersection, ConstraintsUnion, SingleValueConstraint,
+ ValueRangeConstraint, ValueSizeConstraint) = mibBuilder.importSymbols(
+     "ASN1-REFINEMENT", "ConstraintsIntersection", "ConstraintsUnion",
+     "SingleValueConstraint", "ValueRangeConstraint", "ValueSizeConstraint"
+ )
+
 # syntax of objects
 
 OctetString = rfc1902.OctetString
@@ -28,7 +36,7 @@ Opaque = rfc1902.Opaque
 Counter64 = rfc1902.Counter64
 
 class ExtUTCTime(OctetString):
-    subtypeSpec = OctetString.subtypeSpec+ConstraintsUnion(ValueSizeConstraint(11,11), ValueSizeConstraint(13,13))
+    subtypeSpec = OctetString.subtypeSpec+ConstraintsUnion(ValueSizeConstraint(11, 11), ValueSizeConstraint(13, 13))
 
 # MIB tree foundation class
 
@@ -36,13 +44,16 @@ class MibNode:
     label = ''
     def __init__(self, name):
         self.name = name
-        
+
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.name)
 
-    def getName(self): return self.name
-    
-    def getLabel(self): return self.label
+    def getName(self):
+        return self.name
+
+    def getLabel(self):
+        return self.label
+
     def setLabel(self, label):
         self.label = label
         return self
@@ -60,26 +71,35 @@ class MibNode:
 class ModuleIdentity(MibNode):
     def getLastUpdated(self):
         return getattr(self, 'lastUpdated', '')
+
     def setLastUpdated(self, v):
         self.lastUpdated = v
         return self
+
     def getOrganization(self):
         return getattr(self, 'organization', '')
+
     def setOrganization(self, v):
         self.organization = v
         return self
+
     def getContactInfo(self):
         return getattr(self, 'contactInfo', '')
+
     def setContactInfo(self, v):
         self.contactInfo = v
         return self
+
     def getDescription(self):
         return getattr(self, 'description', '')
+
     def setDescription(self, v):
         self.description = v
         return self
+
     def getRevisions(self):
         return getattr(self, 'revisions', ())
+
     def setRevisions(self, args):
         self.revisions = args
         return self
@@ -96,21 +116,26 @@ MODULE-IDENTITY\n\
      self.getOrganization(),
      self.getContactInfo(),
      self.getDescription(),
-     ''.join([ "REVISION \"%s\"\n" % x for x in self.getRevisions() ]))
+     ''.join(["REVISION \"%s\"\n" % x for x in self.getRevisions()]))
 
 class ObjectIdentity(MibNode):
     def getStatus(self):
         return getattr(self, 'status', 'current')
+
     def setStatus(self, v):
         self.status = v
         return self
+
     def getDescription(self):
         return getattr(self, 'description', '')
+
     def setDescription(self, v):
         self.description = v
         return self
+
     def getReference(self):
         return getattr(self, 'reference', '')
+
     def setReference(self, v):
         self.reference = v
         return self
@@ -130,21 +155,28 @@ OBJECT-IDENTITY\n\
 class NotificationType(MibNode):
     def getObjects(self):
         return getattr(self, 'objects', ())
+
     def setObjects(self, *args):
         self.objects = args
         return self
+
     def getStatus(self):
         return getattr(self, 'status', 'current')
+
     def setStatus(self, v):
         self.status = v
         return self
+
     def getDescription(self):
         return getattr(self, 'description', '')
+
     def setDescription(self, v):
         self.description = v
         return self
+
     def getRevisions(self):
         return getattr(self, 'revisions', ())
+
     def setRevisions(self, args):
         self.revisions = args
         return self
@@ -156,10 +188,10 @@ NOTIFICATION-TYPE\n\
   STATUS %s\n\
   DESCRIPTION \"%s\"\n\
   %s\
-' % (', '.join([ x for x in self.getObjects() ]),
+' % (', '.join([x for x in self.getObjects()]),
      self.getStatus(),
      self.getDescription(),
-     ''.join([ "REVISION \"%s\"\n" % x for x in self.getRevisions() ]))
+     ''.join(["REVISION \"%s\"\n" % x for x in self.getRevisions()]))
 
 class MibIdentifier(MibNode):
     def asn1Print(self):
@@ -172,41 +204,67 @@ class ObjectType(MibNode):
         self.syntax = syntax
 
     # XXX
-    def __eq__(self, other): return self.syntax == other
-    def __ne__(self, other): return self.syntax != other
-    def __lt__(self, other): return self.syntax < other
-    def __le__(self, other): return self.syntax <= other
-    def __gt__(self, other): return self.syntax > other
-    def __ge__(self, other): return self.syntax >= other
-    
+    def __eq__(self, other):
+        return self.syntax == other
+
+    def __ne__(self, other):
+        return self.syntax != other
+
+    def __lt__(self, other):
+        return self.syntax < other
+
+    def __le__(self, other):
+        return self.syntax <= other
+
+    def __gt__(self, other):
+        return self.syntax > other
+
+    def __ge__(self, other):
+        return self.syntax >= other
+
     def __repr__(self):
         return '%s(%r, %r)' % (
             self.__class__.__name__, self.name, self.syntax
-            )    
+        )
+
     def getSyntax(self):
         return self.syntax
+
+    def setSyntax(self, v):
+        self.syntax = v
+        return self
+
     def getUnits(self):
         return getattr(self, 'units', '')
+
     def setUnits(self, v):
         self.units = v
-        return self    
+        return self
+
     def getMaxAccess(self):
         return getattr(self, 'maxAccess', 'not-accessible')
+
     def setMaxAccess(self, v):
         self.maxAccess = v
         return self
+
     def getStatus(self):
         return getattr(self, 'status', 'current')
+
     def setStatus(self, v):
         self.status = v
         return self
+
     def getDescription(self):
         return getattr(self, 'description', '')
+
     def setDescription(self, v):
         self.description = v
-        return self    
+        return self
+
     def getReference(self):
         return getattr(self, 'reference', '')
+
     def setReference(self, v):
         self.reference = v
         return self
@@ -226,7 +284,7 @@ OBJECT-TYPE\n\
      self.getStatus(),
      self.getDescription(),
      self.getReference())
-        
+
 class MibTree(ObjectType):
     branchVersionId = 0    # cnanges on tree structure change
     maxAccess = 'not-accessible'
@@ -235,7 +293,7 @@ class MibTree(ObjectType):
         self._vars = OidOrderedDict()
 
     # Subtrees registration
-    
+
     def registerSubtrees(self, *subTrees):
         self.branchVersionId += 1
         for subTree in subTrees:
@@ -260,7 +318,7 @@ class MibTree(ObjectType):
     # Tree traversal
     #
     # Missing branches are indicated by the NoSuchObjectError exception.
-    # Although subtrees may indicate their missing branches by the 
+    # Although subtrees may indicate their missing branches by the
     # NoSuchInstanceError exception.
     #
 
@@ -270,12 +328,13 @@ class MibTree(ObjectType):
             subName = name[:keyLen]
             if subName in self._vars:
                 return self._vars[subName]
-                
+
         raise error.NoSuchObjectError(name=name, idx=idx)
 
     def getNextBranch(self, name, idx=None):
         # Start from the beginning
-        if self._vars: first = list(self._vars.keys())[0]
+        if self._vars:
+            first = list(self._vars.keys())[0]
         if self._vars and name < first:
             return self._vars[first]
         else:
@@ -283,7 +342,7 @@ class MibTree(ObjectType):
                 return self._vars[self._vars.nextKey(name)]
             except KeyError:
                 raise error.NoSuchObjectError(idx=idx, name=name)
- 
+
     def getNode(self, name, idx=None):
         """Return tree node found by name"""
         if name == self.name:
@@ -305,18 +364,18 @@ class MibTree(ObjectType):
                     return self._vars[self._vars.nextKey(nextNode.name)]
                 except KeyError:
                     raise error.NoSuchObjectError(idx=idx, name=name)
- 
+
     # MIB instrumentation
 
     # Read operation
-    
+
     def readTest(self, name, val, idx, acInfo):
         (acFun, acCtx) = acInfo
         if name == self.name:
             if acFun:
-                if self.maxAccess not in (
-                    'readonly', 'readwrite','readcreate'
-                    ) or acFun(name, self.syntax, idx, 'read', acCtx):
+                if self.maxAccess not in ('readonly',
+                                          'readwrite', 'readcreate') or \
+                       acFun(name, self.syntax, idx, 'read', acCtx):
                     raise error.NoAccessError(idx=idx, name=name)
         else:
             try:
@@ -325,7 +384,7 @@ class MibTree(ObjectType):
                 return # missing object is not an error here
             else:
                 node.readTest(name, val, idx, acInfo)
-        
+
     def readGet(self, name, val, idx, acInfo):
         try:
             node = self.getBranch(name, idx)
@@ -335,9 +394,9 @@ class MibTree(ObjectType):
             return node.readGet(name, val, idx, acInfo)
 
     # Read next operation is subtree-specific
-   
+
     depthFirst, breadthFirst = 0, 1
-     
+
     def readTestNext(self, name, val, idx, acInfo, oName=None):
         if oName is None:
             oName = name
@@ -354,7 +413,7 @@ class MibTree(ObjectType):
                 except (error.NoSuchInstanceError, error.NoSuchObjectError):
                     continue
             else:
-                try:                
+                try:
                     node = self.getNextBranch(nextName, idx)
                 except (error.NoSuchInstanceError, error.NoSuchObjectError):
                     if topOfTheMib:
@@ -366,7 +425,7 @@ class MibTree(ObjectType):
                 return node.readTestNext(nextName, val, idx, acInfo, oName)
             except (error.NoAccessError, error.NoSuchInstanceError, error.NoSuchObjectError):
                 pass
-    
+
     def readGetNext(self, name, val, idx, acInfo, oName=None):
         if oName is None:
             oName = name
@@ -375,7 +434,7 @@ class MibTree(ObjectType):
             topOfTheMib = False
         nextName = name
         direction = self.depthFirst
-        while 1:  # XXX linear search here
+        while True:  # XXX linear search ahead!
             if direction == self.depthFirst:
                 direction = self.breadthFirst
                 try:
@@ -397,9 +456,9 @@ class MibTree(ObjectType):
                 pass
 
     # Write operation
-    
+
     def writeTest(self, name, val, idx, acInfo):
-        (acFun, acCtx) = acInfo
+        acFun, acCtx = acInfo
         if name == self.name:
             # Make sure variable is writable
             if acFun:
@@ -409,14 +468,14 @@ class MibTree(ObjectType):
         else:
             node = self.getBranch(name, idx)
             node.writeTest(name, val, idx, acInfo)
-    
+
     def writeCommit(self, name, val, idx, acInfo):
         self.getBranch(name, idx).writeCommit(name, val, idx, acInfo)
-    
+
     def writeCleanup(self, name, val, idx, acInfo):
         self.branchVersionId += 1
         self.getBranch(name, idx).writeCleanup(name, val, idx, acInfo)
-    
+
     def writeUndo(self, name, val, idx, acInfo):
         self.getBranch(name, idx).writeUndo(name, val, idx, acInfo)
 
@@ -452,20 +511,20 @@ class MibScalar(MibTree):
         try:
             return MibTree.getNextNode(self, name, idx)
         except (error.NoSuchInstanceError, error.NoSuchObjectError):
-            raise error.NoSuchInstanceError(idx=idx, name=name)       
+            raise error.NoSuchInstanceError(idx=idx, name=name)
 
     # MIB instrumentation methods
-    
+
     # Read operation
- 
+
     def readTest(self, name, val, idx, acInfo):
         (acFun, acCtx) = acInfo
         if name == self.name:
             raise error.NoAccessError(idx=idx, name=name)
         if acFun:
-            if self.maxAccess not in (
-                'readonly', 'readwrite', 'readcreate'
-                ) or acFun(name, self.syntax, idx, 'read', acCtx):
+            if self.maxAccess not in ('readonly', 'readwrite',
+                                      'readcreate') or \
+                    acFun(name, self.syntax, idx, 'read', acCtx):
                 raise error.NoAccessError(idx=idx, name=name)
         MibTree.readTest(self, name, val, idx, acInfo)
 
@@ -480,28 +539,28 @@ class MibScalar(MibTree):
     def readTestNext(self, name, val, idx, acInfo, oName=None):
         (acFun, acCtx) = acInfo
         if acFun:
-            if self.maxAccess not in (
-                'readonly', 'readwrite', 'readcreate'
-                ) or acFun(name, self.syntax, idx, 'read', acCtx):
+            if self.maxAccess not in ('readonly', 'readwrite',
+                                      'readcreate') or \
+                    acFun(name, self.syntax, idx, 'read', acCtx):
                 raise error.NoAccessError(idx=idx, name=name)
         MibTree.readTestNext(self, name, val, idx, acInfo, oName)
- 
+
     def readGetNext(self, name, val, idx, acInfo, oName=None):
         (acFun, acCtx) = acInfo
         # have to dublicate AC here as *Next code above treats
         # noAccess as a noSuchObject at the Test stage, goes on
         # to Reading
         if acFun:
-            if self.maxAccess not in (
-                'readonly', 'readwrite', 'readcreate'
-                ) or acFun(name, self.syntax, idx, 'read', acCtx):
+            if self.maxAccess not in ('readonly', 'readwrite',
+                                      'readcreate') or \
+                    acFun(name, self.syntax, idx, 'read', acCtx):
                 raise error.NoAccessError(idx=idx, name=name)
         return MibTree.readGetNext(self, name, val, idx, acInfo, oName)
- 
+
     # Two-phase commit implementation
 
     def writeTest(self, name, val, idx, acInfo):
-        (acFun, acCtx) = acInfo
+        acFun, acCtx = acInfo
         if name == self.name:
             raise error.NoAccessError(idx=idx, name=name)
         if acFun:
@@ -513,7 +572,7 @@ class MibScalar(MibTree):
 class MibScalarInstance(MibTree):
     """Scalar MIB variable instance. Implements read/write operations."""
     def __init__(self, typeName, instId, syntax):
-        MibTree.__init__(self, typeName+instId, syntax)
+        MibTree.__init__(self, typeName + instId, syntax)
         self.typeName = typeName
         self.instId = instId
         self.__oldSyntax = None
@@ -568,21 +627,21 @@ class MibScalarInstance(MibTree):
         raise error.NoSuchInstanceError(idx=idx, name=name)
 
     # MIB instrumentation methods
-    
+
     # Read operation
-    
+
     def readTest(self, name, val, idx, acInfo):
         if name != self.name:
             raise error.NoSuchInstanceError(idx=idx, name=name)
 
     def readGet(self, name, val, idx, acInfo):
-        # Return current variable (name, value). 
+        # Return current variable (name, value)
         if name == self.name:
             debug.logger & debug.flagIns and debug.logger('readGet: %s=%r' % (self.name, self.syntax))
             return self.name, self.getValue(name, idx)
         else:
             raise error.NoSuchInstanceError(idx=idx, name=name)
- 
+
     def readTestNext(self, name, val, idx, acInfo, oName=None):
         if name != self.name or name <= oName:
             raise error.NoSuchInstanceError(idx=idx, name=name)
@@ -593,7 +652,7 @@ class MibScalarInstance(MibTree):
             return self.readGet(name, val, idx, acInfo)
         else:
             raise error.NoSuchInstanceError(idx=idx, name=name)
-    
+
     # Write operation: two-phase commit
 
     def writeTest(self, name, val, idx, acInfo):
@@ -616,15 +675,15 @@ class MibScalarInstance(MibTree):
         # Backup original value
         if self.__oldSyntax is None:
             self.__oldSyntax = self.syntax
-        # Commit new value            
+        # Commit new value
         self.syntax = self.__newSyntax
-        
+
     def writeCleanup(self, name, val, idx, acInfo):
         self.branchVersionId += 1
         debug.logger & debug.flagIns and debug.logger('writeCleanup: %s=%r' % (name, val))
         # Drop previous value
         self.__newSyntax = self.__oldSyntax = None
-    
+
     def writeUndo(self, name, val, idx, acInfo):
         # Revive previous value
         self.syntax = self.__oldSyntax
@@ -675,10 +734,14 @@ class MibScalarInstance(MibTree):
                     self.__newSyntax = why['syntax']
         else:
             raise error.NoSuchInstanceError(idx=idx, name=name)
-    def destroyCommit(self, name, val, idx, acInfo): pass
+    def destroyCommit(self, name, val, idx, acInfo):
+        pass
+
     def destroyCleanup(self, name, val, idx, acInfo):
         self.branchVersionId += 1
-    def destroyUndo(self, name, val, idx, acInfo): pass
+
+    def destroyUndo(self, name, val, idx, acInfo):
+        pass
 
 # Conceptual table classes
 
@@ -687,9 +750,10 @@ class MibTableColumn(MibScalar):
     protoInstance = MibScalarInstance
     def __init__(self, name, syntax):
         MibScalar.__init__(self, name, syntax)
-        self.__createdInstances = {}; self.__destroyedInstances = {}
+        self.__createdInstances = {}
+        self.__destroyedInstances = {}
         self.__rowOpWanted = {}
-    
+
     #
     # Subtree traversal
     #
@@ -707,7 +771,7 @@ class MibTableColumn(MibScalar):
     # Column creation (this should probably be converted into some state
     # machine for clarity). Also, it might be a good idea to inidicate
     # defaulted cols creation in a clearer way than just a val == None.
-    
+
     def createTest(self, name, val, idx, acInfo):
         (acFun, acCtx) = acInfo
         # Make sure creation allowed, create a new column instance but
@@ -725,9 +789,9 @@ class MibTableColumn(MibScalar):
             return
         self.__createdInstances[name] = self.protoInstance(
             self.name, name[len(self.name):], self.syntax.clone()
-            )
+        )
         self.__createdInstances[name].createTest(name, val, idx, acInfo)
-            
+
     def createCommit(self, name, val, idx, acInfo):
         # Commit new instance value
         if name in self._vars: # XXX
@@ -744,9 +808,8 @@ class MibTableColumn(MibScalar):
         self.branchVersionId += 1
         if name in self.__createdInstances:
             if self.__createdInstances[name] is not None:
-                self.__createdInstances[name].createCleanup(
-                    name, val, idx, acInfo
-                    )
+                self.__createdInstances[name].createCleanup(name, val, idx,
+                                                            acInfo)
             del self.__createdInstances[name]
         elif name in self._vars:
             self._vars[name].createCleanup(name, val, idx, acInfo)
@@ -767,14 +830,14 @@ class MibTableColumn(MibScalar):
                     del self._vars[name]
                 else:
                     self._vars[name].createUndo(name, val, idx, acInfo)
-                
+
     # Column destruction
-        
+
     def destroyTest(self, name, val, idx, acInfo):
         (acFun, acCtx) = acInfo
         # Make sure destruction is allowed
         if name == self.name:
-            raise error.NoAccessError(idx=idx, name=name)        
+            raise error.NoAccessError(idx=idx, name=name)
         if name not in self._vars:
             return
         if acFun:
@@ -786,41 +849,40 @@ class MibTableColumn(MibScalar):
     def destroyCommit(self, name, val, idx, acInfo):
         # Make a copy of column instance and take it off the tree
         if name in self._vars:
-            self._vars[name].destroyCommit(name, val, idx, acInfo)          
+            self._vars[name].destroyCommit(name, val, idx, acInfo)
             self.__destroyedInstances[name] = self._vars[name]
             del self._vars[name]
-        
+
     def destroyCleanup(self, name, val, idx, acInfo):
         # Drop instance copy
         self.branchVersionId += 1
         if name in self.__destroyedInstances:
-            self.__destroyedInstances[name].destroyCleanup(
-                name, val, idx, acInfo
-                )
+            self.__destroyedInstances[name].destroyCleanup(name, val,
+                                                           idx, acInfo)
             debug.logger & debug.flagIns and debug.logger('destroyCleanup: %s=%r' % (name, val))
             del self.__destroyedInstances[name]
-            
+
     def destroyUndo(self, name, val, idx, acInfo):
         # Set back column instance
         if name in self.__destroyedInstances:
             self._vars[name] = self.__destroyedInstances[name]
-            self._vars[name].destroyUndo(
-                name, val, idx, acInfo
-                )            
+            self._vars[name].destroyUndo(name, val, idx, acInfo)
             del self.__destroyedInstances[name]
-            
+
     # Set/modify column
 
     def writeTest(self, name, val, idx, acInfo):
         # Besides common checks, request row creation on no-instance
         try:
             # First try the instance
-            MibScalar.writeTest(
-                self, name, val, idx, acInfo
-                )
+            MibScalar.writeTest(self, name, val, idx, acInfo)
         # ...otherwise proceed with creating new column
         except (error.NoSuchInstanceError, error.RowCreationWanted):
-            self.__rowOpWanted[name] = error.RowCreationWanted()
+            excValue = sys.exc_info()[1]
+            if isinstance(excValue, error.RowCreationWanted):
+                self.__rowOpWanted[name] = excValue
+            else:
+                self.__rowOpWanted[name] = error.RowCreationWanted()
             self.createTest(name, val, idx, acInfo)
         except error.RowDestructionWanted:
             self.__rowOpWanted[name] = error.RowDestructionWanted()
@@ -831,41 +893,31 @@ class MibTableColumn(MibScalar):
 
     def __delegateWrite(self, subAction, name, val, idx, acInfo):
         if name not in self.__rowOpWanted:
-            getattr(MibScalar, 'write'+subAction)(
-                self, name, val, idx, acInfo
-                )
+            getattr(MibScalar, 'write'+subAction)(self, name, val, idx, acInfo)
             return
         if isinstance(self.__rowOpWanted[name], error.RowCreationWanted):
-            getattr(self, 'create'+subAction)(
-                name, val, idx, acInfo
-                )
+            getattr(self, 'create'+subAction)(name, val, idx, acInfo)
         if isinstance(self.__rowOpWanted[name], error.RowDestructionWanted):
-            getattr(self, 'destroy'+subAction)(
-                           name, val, idx, acInfo
-                )
-        
+            getattr(self, 'destroy'+subAction)(name, val, idx, acInfo)
+
     def writeCommit(self, name, val, idx, acInfo):
-        self.__delegateWrite(
-            'Commit', name, val, idx, acInfo
-            )
+        self.__delegateWrite('Commit', name, val, idx, acInfo)
         if name in self.__rowOpWanted:
             raise self.__rowOpWanted[name]
 
     def writeCleanup(self, name, val, idx, acInfo):
         self.branchVersionId += 1
-        self.__delegateWrite(
-            'Cleanup', name, val, idx, acInfo
-            )
+        self.__delegateWrite('Cleanup', name, val, idx, acInfo)
         if name in self.__rowOpWanted:
             e = self.__rowOpWanted[name]
             del self.__rowOpWanted[name]
             debug.logger & debug.flagIns and debug.logger('%s dropped by %s=%r' % (e, name, val))
             raise e
-            
+
     def writeUndo(self, name, val, idx, acInfo):
-        self.__delegateWrite(
-            'Undo', name, val, idx, acInfo
-            )
+        if name in self.__rowOpWanted:
+            self.__rowOpWanted[name] = error.RowDestructionWanted()
+        self.__delegateWrite('Undo', name, val, idx, acInfo)
         if name in self.__rowOpWanted:
             e = self.__rowOpWanted[name]
             del self.__rowOpWanted[name]
@@ -927,7 +979,7 @@ class MibTableRow(MibTree):
         baseTag = obj.getTagSet().getBaseTag()
         if baseTag == self.__intBaseTag:
             return (int(obj),)
-        elif self.__ipaddrTagSet.isSuperTagSetOf(obj.getTagSet()):        
+        elif self.__ipaddrTagSet.isSuperTagSetOf(obj.getTagSet()):
             return obj.asNumbers()
         elif baseTag == self.__strBaseTag:
             if impliedFlag or obj.isFixedLength():
@@ -942,10 +994,10 @@ class MibTableRow(MibTree):
                 return (len(self.name),) + tuple(obj)
         # rfc2578, 7.1
         elif baseTag == self.__bitsBaseTag:
-            return ( len(obj), ) +  obj.asNumbers()
+            return (len(obj),) +  obj.asNumbers()
         else:
             raise error.SmiError('Unknown value type for index %r' % (obj,))
-            
+
     # Fate sharing mechanics
 
     def announceManagementEvent(self, action, name, val, idx, acInfo):
@@ -954,25 +1006,25 @@ class MibTableRow(MibTree):
         baseIndices = []
         for impliedFlag, modName, symName in self.indexNames:
             mibObj, = mibBuilder.importSymbols(modName, symName)
-            syntax, instId = self.setFromName(
-                mibObj.syntax, instId, impliedFlag
-                )
+            syntax, instId = self.setFromName(mibObj.syntax, instId,
+                                              impliedFlag)
+
             if self.name == mibObj.name[:-1]:
                 baseIndices.append((mibObj.name, syntax))
+
         if instId:
-            raise error.SmiError(
-                'Excessive instance identifier sub-OIDs left at %s: %s' %
-                (self, instId)
-                )
+            raise error.SmiError('Excessive instance identifier sub-OIDs left at %s: %s' % (self, instId))
+
         if not baseIndices:
             return
+
         for modName, mibSym in self.augmentingRows.keys():
             mibObj, = mibBuilder.importSymbols(modName, mibSym)
             debug.logger & debug.flagIns and debug.logger('announceManagementEvent %s to %s' % (action, mibObj))
             mibObj.receiveManagementEvent(
                 action, baseIndices, val, idx, acInfo
-                )
-            
+            )
+
     def receiveManagementEvent(self, action, baseIndices, val, idx, acInfo):
         # The default implementation supports one-to-one rows dependency
         newSuffix = ()
@@ -982,6 +1034,7 @@ class MibTableRow(MibTree):
             for name, syntax in baseIndices:
                 if name == mibObj.name:
                     newSuffix = newSuffix + self.getAsName(syntax, impliedFlag)
+
         if newSuffix:
             debug.logger & debug.flagIns and debug.logger('receiveManagementEvent %s for suffix %s' % (action, newSuffix))
             self.__manageColumns(action, (), newSuffix, val, idx, acInfo)
@@ -992,10 +1045,10 @@ class MibTableRow(MibTree):
                 raise error.SmiError(
                     'Row %s already augmented by %s::%s' % \
                     (self.name, modName, symName)
-                    )
+                )
             self.augmentingRows[(modName, symName)] = 1
         return self
-        
+
     def setIndexNames(self, *names):
         for name in names:
             self.indexNames = self.indexNames + (name,)
@@ -1003,62 +1056,83 @@ class MibTableRow(MibTree):
 
     def getIndexNames(self):
         return self.indexNames
-                             
+
     def __manageColumns(self, action, excludeName, nameSuffix,
                         val, idx, acInfo):
         # Build a map of index names and values for automatic initialization
-        indexVals = {}; instId = nameSuffix
+        indexVals = {}
+        instId = nameSuffix
         for impliedFlag, modName, symName in self.indexNames:
             mibObj, = mibBuilder.importSymbols(modName, symName)
-            syntax, instId = self.setFromName(
-                mibObj.syntax, instId, impliedFlag
-                )
+            syntax, instId = self.setFromName(mibObj.syntax, instId,
+                                              impliedFlag)
             indexVals[mibObj.name] = syntax
+
         for name, var in self._vars.items():
             if name == excludeName:
                 continue
+
             if name in indexVals:
                 getattr(var, action)(name + nameSuffix, indexVals[name], idx,
                                      (None, None))
             else:
                 getattr(var, action)(name + nameSuffix, val, idx, acInfo)
+
             debug.logger & debug.flagIns and debug.logger('__manageColumns: action %s name %s suffix %s %svalue %r' % (action, name, nameSuffix, name in indexVals and "index " or "", indexVals.get(name, val)))
 
     def __delegate(self, subAction, name, val, idx, acInfo):
         # Relay operation request to column, expect row operation request.
+        rowIsActive = False
         try:
             getattr(self.getBranch(name, idx), 'write'+subAction)(
                 name, val, idx, acInfo
-                )
+            )
+
         except error.RowCreationWanted:
             self.__manageColumns(
                 'create'+subAction, name[:len(self.name)+1],
                 name[len(self.name)+1:], None, idx, acInfo
-                )
+            )
+
             self.announceManagementEvent(
                 'create'+subAction, name, None, idx, acInfo
-                )
+            )
+
+            # watch for RowStatus == 'stActive'
+            rowIsActive = sys.exc_info()[1].get('syntax', 0) == 1
+
         except error.RowDestructionWanted:
             self.__manageColumns(
                 'destroy'+subAction, name[:len(self.name)+1],
                 name[len(self.name)+1:], None, idx, acInfo
-                )
+            )
+
             self.announceManagementEvent(
                 'destroy'+subAction, name, None, idx, acInfo
-                )
-    
+            )
+
+        return rowIsActive
+
     def writeTest(self, name, val, idx, acInfo):
         self.__delegate('Test', name, val, idx, acInfo)
+
     def writeCommit(self, name, val, idx, acInfo):
-        self.__delegate('Commit', name, val, idx, acInfo)
+        rowIsActive = self.__delegate('Commit', name, val, idx, acInfo)
+        if rowIsActive:
+            for mibNode in self._vars.values():
+                colNode = mibNode.getNode(mibNode.name + name[len(self.name)+1:])
+                if not colNode.syntax.hasValue():
+                    raise error.InconsistentValueError(msg='Row consistency check failed for %r' % colNode)
+
     def writeCleanup(self, name, val, idx, acInfo):
         self.branchVersionId += 1
         self.__delegate('Cleanup', name, val, idx, acInfo)
-    def writeUndo(self, name, val,  idx, acInfo):
+
+    def writeUndo(self, name, val, idx, acInfo):
         self.__delegate('Undo', name, val, idx, acInfo)
 
     # Table row management
-    
+
     # Table row access by instance name
 
     def getInstName(self, colId, instId):
@@ -1079,7 +1153,7 @@ class MibTableRow(MibTree):
             raise error.SmiError(
                 'Excessive instance identifier sub-OIDs left at %s: %s' %
                 (self, instId)
-                )
+            )
         indices = tuple(indices)
         self.__idToIdxCache[instId] = indices
         return indices
@@ -1088,13 +1162,15 @@ class MibTableRow(MibTree):
         """Return column instance identification from indices"""
         if indices in self.__idxToIdCache:
             return self.__idxToIdCache[indices]
-        idx = 0; idxLen = len(indices); instId = ()
+        idx = 0
+        idxLen = len(indices)
+        instId = ()
         for impliedFlag, modName, symName in self.indexNames:
             mibObj, = mibBuilder.importSymbols(modName, symName)
             if idx < idxLen:
                 instId = instId + self.getAsName(
                     mibObj.syntax.clone(indices[idx]), impliedFlag
-                    )
+                )
             else:
                 break
             idx = idx + 1
@@ -1102,7 +1178,7 @@ class MibTableRow(MibTree):
         return instId
 
     # Table access by index
-    
+
     def getInstNameByIndex(self, colId, *indices):
         """Build column instance name from components"""
         return self.name + (colId,) + self.getInstIdFromIndices(*indices)
@@ -1113,19 +1189,21 @@ class MibTableRow(MibTree):
         for columnName in self._vars.keys():
             instNames.append(
                 self.getInstNameByIndex(*(columnName[-1],) + indices)
-                )
-                
+            )
+
         return tuple(instNames)
-    
+
 class MibTable(MibTree):
     """MIB table. Manages a set of TableRow's"""
     def __init__(self, name):
         MibTree.__init__(self, name)
-        
-zeroDotZero = ObjectIdentity((0,0))
 
-#dot = MibTree()
+zeroDotZero = ObjectIdentity((0, 0))
+
+# OID tree
+itu_t = MibTree((0,)).setLabel('itu-t')
 iso = MibTree((1,))
+joint_iso_itu_t = MibTree((2,)).setLabel('joint-iso-itu-t')
 org = MibIdentifier(iso.name + (3,))
 dod = MibIdentifier(org.name + (6,))
 internet = MibIdentifier(dod.name + (1,))
@@ -1146,7 +1224,7 @@ snmpModules = MibIdentifier(snmpV2.name +(3,))
 mibBuilder.exportSymbols(
     'SNMPv2-SMI', MibNode=MibNode,
     Integer32=Integer32, Bits=Bits, IpAddress=IpAddress,
-    Counter32=Counter32,    Gauge32=Gauge32, Unsigned32=Unsigned32,
+    Counter32=Counter32, Gauge32=Gauge32, Unsigned32=Unsigned32,
     TimeTicks=TimeTicks, Opaque=Opaque, Counter64=Counter64,
     ExtUTCTime=ExtUTCTime,
     ModuleIdentity=ModuleIdentity, ObjectIdentity=ObjectIdentity,
@@ -1155,12 +1233,12 @@ mibBuilder.exportSymbols(
     MibIdentifier=MibIdentifier, MibTree=MibTree,
     MibTableColumn=MibTableColumn, MibTableRow=MibTableRow,
     MibTable=MibTable, zeroDotZero=zeroDotZero,
-    iso=iso, org=org, dod=dod, internet=internet,
-    directory=directory, mgmt=mgmt, mib_2=mib_2, transmission=transmission,
-    experimental=experimental, private=private, enterprises=enterprises,
-    security=security, snmpV2=snmpV2, snmpDomains=snmpDomains,
-    snmpProxys=snmpProxys, snmpModules=snmpModules    
-    )
+    itu_t=itu_t, iso=iso, joint_iso_itu_t=joint_iso_itu_t, org=org, dod=dod,
+    internet=internet, directory=directory, mgmt=mgmt, mib_2=mib_2,
+    transmission=transmission, experimental=experimental, private=private,
+    enterprises=enterprises, security=security, snmpV2=snmpV2,
+    snmpDomains=snmpDomains, snmpProxys=snmpProxys, snmpModules=snmpModules
+)
 
 # XXX
 # getAsName/setFromName goes out of MibRow?
